@@ -77,7 +77,7 @@ class KarakeepAPI:
 
     Attributes:
         api_key (str): The API key used for authentication.
-        api_base_url (str): The base URL of the Karakeep API instance.
+        api_endpoint (str): The endpoint of the Karakeep API instance, including /api/v1 (e.g., https://instance.com/api/v1/).
         openapi_spec (dict): The parsed content of the OpenAPI specification file.
         verify_ssl (bool): Whether SSL verification is enabled.
         verbose (bool): Whether verbose logging is enabled.
@@ -85,12 +85,12 @@ class KarakeepAPI:
     """
 
     # Version reflects the client library version, updated by bumpver
-    VERSION: str = "1.0.0"
+    VERSION: str = "1.2.2"
 
     def __init__(
         self,
         api_key: Optional[str] = None,
-        base_url: Optional[str] = None,
+        api_endpoint: Optional[str] = None,
         openapi_spec_path: Optional[str] = None,  # Allow None, default handled below
         verify_ssl: bool = True,
         verbose: bool = False,
@@ -105,8 +105,9 @@ class KarakeepAPI:
         Args:
             api_key: Karakeep API key (Bearer token).
                      Defaults to KARAKEEP_PYTHON_API_KEY environment variable if not provided.
-            base_url: Override the base URL for the API. Must be provided either as an argument
-                      or via the KARAKEEP_PYTHON_API_BASE_URL environment variable.
+            api_endpoint: Override the endpoint for the API. Must be provided either as an argument
+                          or via the KARAKEEP_PYTHON_API_ENDPOINT environment variable.
+                          Example: 'https://karakeep.domain.com/api/v1/'
             openapi_spec_path: Path to the OpenAPI JSON specification file.
                                Defaults to 'openapi_reference.json' alongside the package code if not provided.
                                The loaded spec is available via the `openapi_spec` attribute.
@@ -130,47 +131,47 @@ class KarakeepAPI:
         self.api_key = resolved_api_key
         logger.debug("API Key loaded successfully.")
 
-        # --- Base URL Validation ---
-        env_base_url = os.environ.get("KARAKEEP_PYTHON_API_BASE_URL")
+        # --- Endpoint Validation ---
+        env_endpoint = os.environ.get("KARAKEEP_PYTHON_API_ENDPOINT")
         logger.debug(
-            f"Checked KARAKEEP_PYTHON_API_BASE_URL environment variable, found: '{env_base_url}'"
+            f"Checked KARAKEEP_PYTHON_API_ENDPOINT environment variable, found: '{env_endpoint}'"
         )
-        logger.debug(f"Base URL provided as argument: '{base_url}'")
+        logger.debug(f"Endpoint provided as argument: '{api_endpoint}'")
 
-        if base_url:
-            self.api_base_url = base_url
-            logger.info(f"Using provided base URL: {self.api_base_url}")
-        elif env_base_url:
-            self.api_base_url = env_base_url
+        if api_endpoint:
+            self.api_endpoint = api_endpoint
+            logger.info(f"Using provided endpoint: {self.api_endpoint}")
+        elif env_endpoint:
+            self.api_endpoint = env_endpoint
             logger.info(
-                f"Using base URL from KARAKEEP_PYTHON_API_BASE_URL: {self.api_base_url}"
+                f"Using endpoint from KARAKEEP_PYTHON_API_ENDPOINT: {self.api_endpoint}"
             )
         else:
-            # No base_url from arg or env var - raise error as per requirement
+            # No api_endpoint from arg or env var - raise error as per requirement
             raise ValueError(
-                "API base URL is required. Provide 'base_url' argument or set KARAKEEP_PYTHON_API_BASE_URL environment variable."
+                "API endpoint is required. Provide 'api_endpoint' argument or set KARAKEEP_PYTHON_API_ENDPOINT environment variable."
             )
 
-        # Ensure base URL ends with /v1/
-        resolved_url = self.api_base_url  # Use a temporary variable for checks
-        if resolved_url.endswith("/v1"):
-            # Ends with /v1, needs a slash
-            self.api_base_url = resolved_url + "/"
+        # Ensure endpoint ends with /api/v1/
+        resolved_url = self.api_endpoint  # Use a temporary variable for checks
+        if resolved_url.endswith("/api/v1"):
+            # Ends with /api/v1, needs a slash
+            self.api_endpoint = resolved_url + "/"
             logger.info(
-                f"Appended trailing slash to base URL ending in /v1: {self.api_base_url}"
+                f"Appended trailing slash to endpoint ending in /api/v1: {self.api_endpoint}"
             )
-        elif resolved_url.endswith("/v1/"):
+        elif resolved_url.endswith("/api/v1/"):
             # Already ends correctly, do nothing
-            logger.debug(f"Base URL already ends with /v1/: {self.api_base_url}")
+            logger.debug(f"Endpoint already ends with /api/v1/: {self.api_endpoint}")
         else:
-            # Doesn't end with /v1 or /v1/, append /v1/
-            # First, remove any existing trailing slash to avoid //v1/
+            # Doesn't end with /api/v1 or /api/v1/, append /api/v1/
+            # First, remove any existing trailing slash to avoid //api/v1/
             if resolved_url.endswith("/"):
                 resolved_url = resolved_url[:-1]
-            self.api_base_url = resolved_url + "/v1/"
-            logger.info(f"Appended /v1/ to base URL: {self.api_base_url}")
+            self.api_endpoint = resolved_url + "/api/v1/"
+            logger.info(f"Appended /api/v1/ to endpoint: {self.api_endpoint}")
 
-        logger.debug(f"Final API Base URL after /v1/ check: {self.api_base_url}")
+        logger.debug(f"Final API Endpoint after /api/v1/ check: {self.api_endpoint}")
 
         # --- Load and Parse OpenAPI Spec ---
         if openapi_spec_path is None:
@@ -257,7 +258,7 @@ class KarakeepAPI:
         # self.verbose is still used for conditional logging within the class methods.
 
         logger.debug("KarakeepAPI client initialized.")
-        logger.debug(f"  Base URL: {self.api_base_url}")
+        logger.debug(f"  Endpoint: {self.api_endpoint}")
         logger.debug(f"  Verify SSL: {self.verify_ssl}")
         logger.debug(f"  Verbose: {self.verbose}")
         logger.debug(
@@ -294,34 +295,37 @@ class KarakeepAPI:
         data: Optional[
             Union[BaseModel, dict, list, str, bytes]
         ] = None,  # More specific type hint
+        files: Optional[Dict[str, Any]] = None,
         extra_headers: Optional[Dict[str, str]] = None,
-    ) -> Union[Dict[str, Any], List[Any], None]:
+    ) -> Union[Dict[str, Any], List[Any], None, bytes]:
         """
         Internal method to make an HTTP call to the Karakeep API. Handles authentication,
         request formatting, response parsing, and error handling.
 
         Args:
             method: HTTP method ('GET', 'POST', 'PUT', 'PATCH', 'DELETE').
-            endpoint: API endpoint path relative to the base URL (e.g., 'bookmarks' or 'bookmarks/some_id').
+            endpoint: API endpoint path relative to the endpoint (e.g., 'bookmarks' or 'bookmarks/some_id').
                       Path parameters (like {bookmarkId}) MUST be substituted *before* calling _call.
             params: Dictionary of URL query parameters. Values should be primitive types suitable for URLs.
             data: Request body data. Can be a Pydantic model, dict, list, bytes, or str.
                   - Pydantic models, dicts, and lists will be automatically JSON-encoded
                     with 'Content-Type: application/json' unless overridden in extra_headers.
                   - For bytes or str, ensure 'Content-Type' is set correctly via extra_headers if needed.
+            files: Dictionary for file uploads (multipart/form-data). If provided, data parameter is ignored.
             extra_headers: Additional headers to include or override default headers.
 
         Returns:
-            The parsed JSON response from the API as a dict or list, or None for 204 No Content responses.
-            The calling wrapper method is responsible for further parsing/validation into specific Pydantic models.
+            The parsed JSON response from the API as a dict or list, None for 204 No Content responses,
+            or raw bytes for non-JSON responses. The calling wrapper method is responsible for further
+            parsing/validation into specific Pydantic models.
 
         Raises:
             AuthenticationError: If authentication fails (401).
             APIError: For other HTTP errors or request issues.
         """
-        # Ensure endpoint doesn't start with / if base_url ends with /
+        # Ensure endpoint doesn't start with / if endpoint ends with /
         safe_endpoint = endpoint.lstrip("/")
-        url = urljoin(self.api_base_url, safe_endpoint)
+        url = urljoin(self.api_endpoint, safe_endpoint)
 
         # Default headers
         headers = {
@@ -345,7 +349,16 @@ class KarakeepAPI:
         # Determine Content-Type, prioritizing extra_headers
         content_type = headers.get("Content-Type")
 
-        if data is not None:
+        # Handle file uploads (multipart/form-data)
+        if files is not None:
+            # When files are provided, let requests handle Content-Type automatically
+            # Don't set Content-Type header for multipart uploads
+            if "Content-Type" in headers:
+                # Remove Content-Type if it was set, let requests set it for multipart
+                headers.pop("Content-Type")
+            # Don't process data when files are provided
+            request_body_arg = None
+        elif data is not None:
             if isinstance(data, BaseModel):
                 # Serialize Pydantic model to JSON bytes
                 request_body_arg = data.model_dump_json(
@@ -470,6 +483,7 @@ class KarakeepAPI:
                         url=url,
                         params=request_params,  # Use params with stringified booleans
                         data=request_body_arg,  # Serialized data (bytes or str)
+                        files=files,  # File uploads for multipart/form-data
                         headers=headers,
                         verify=self.verify_ssl,
                         timeout=60,  # Increased default timeout
@@ -514,28 +528,42 @@ class KarakeepAPI:
                     logger.debug("  Body: None (204 No Content or empty response body)")
                 return None
 
-            # Attempt to parse successful response as JSON
-            try:
-                result = response.json()
+            # Check if the response is expected to be JSON based on Accept header
+            accept_header = headers.get("Accept", "application/json")
+            expects_json = "application/json" in accept_header
+
+            # Attempt to parse successful response as JSON if we expect JSON
+            if expects_json:
+                try:
+                    result = response.json()
+                    if self.verbose:
+                        # Log parsed response body carefully
+                        log_resp_str = repr(result)
+                        if len(log_resp_str) > 1000:
+                            log_resp_str = log_resp_str[:1000] + "...(truncated)"
+                        logger.debug(f"  Body (JSON Parsed): {log_resp_str}")
+                    # Return the raw parsed JSON (dict/list). Deserialization into
+                    # specific Pydantic models should happen in the calling wrapper method.
+                    return result
+                except json.JSONDecodeError as e:
+                    # Handle cases where the response is successful (2xx) but not valid JSON
+                    logger.error(
+                        f"API Error: Failed to decode JSON response from {method} {url}. Status: {response.status_code}. Content: {response.text[:500]}..."
+                    )
+                    # Raise APIError as the response format is unexpected
+                    raise APIError(
+                        message=f"Failed to parse successful API response JSON from {url}: {e}. Response text: {response.text[:200]}...",
+                        status_code=response.status_code,
+                    ) from e
+            else:
+                # For non-JSON responses (like asset downloads), return raw bytes
                 if self.verbose:
-                    # Log parsed response body carefully
-                    log_resp_str = repr(result)
-                    if len(log_resp_str) > 1000:
-                        log_resp_str = log_resp_str[:1000] + "...(truncated)"
-                    logger.debug(f"  Body (JSON Parsed): {log_resp_str}")
-                # Return the raw parsed JSON (dict/list). Deserialization into
-                # specific Pydantic models should happen in the calling wrapper method.
-                return result
-            except json.JSONDecodeError as e:
-                # Handle cases where the response is successful (2xx) but not valid JSON
-                logger.error(
-                    f"API Error: Failed to decode JSON response from {method} {url}. Status: {response.status_code}. Content: {response.text[:500]}..."
-                )
-                # Raise APIError as the response format is unexpected
-                raise APIError(
-                    message=f"Failed to parse successful API response JSON from {url}: {e}. Response text: {response.text[:200]}...",
-                    status_code=response.status_code,
-                ) from e
+                    content_type = response.headers.get("Content-Type", "unknown")
+                    content_length = len(response.content)
+                    logger.debug(
+                        f"  Body (Binary): {content_length} bytes, Content-Type: {content_type}"
+                    )
+                return response.content
 
         except requests.exceptions.HTTPError as e:
             # Handle 4xx/5xx errors raised by response.raise_for_status()
@@ -1179,7 +1207,7 @@ class KarakeepAPI:
             "precrawledArchive",
             "unknown",
         ],
-    ) -> Union[datatypes.Asset, Dict[str, Any], List[Any]]:
+    ) -> Union[datatypes.BookmarkAsset, Dict[str, Any], List[Any]]:
         """
         Attach a new asset to a bookmark. Corresponds to POST /bookmarks/{bookmarkId}/assets.
 
@@ -1190,7 +1218,7 @@ class KarakeepAPI:
                         "bannerImage", "fullPageArchive", "video", "bookmarkAsset", "precrawledArchive", "unknown".
 
         Returns:
-            datatypes.Asset: The attached asset object.
+            datatypes.BookmarkAsset: The attached asset object.
             If response validation is disabled, returns the raw API response (dict/list).
 
         Raises:
@@ -1207,8 +1235,8 @@ class KarakeepAPI:
             logger.debug("Skipping response validation as requested.")
             return response_data
         else:
-            # Response should match Asset schema
-            return datatypes.Asset.model_validate(response_data)
+            # Response should match BookmarkAsset schema
+            return datatypes.BookmarkAsset.model_validate(response_data)
 
     @optional_typecheck
     def replace_asset(self, bookmark_id: str, asset_id: str, new_asset_id: str) -> None:
@@ -1306,6 +1334,7 @@ class KarakeepAPI:
         parent_id: Optional[str] = None,
         list_type: Optional[Literal["manual", "smart"]] = "manual",
         query: Optional[str] = None,
+        public: bool = False,
     ) -> Union[datatypes.ListModel, Dict[str, Any], List[Any]]:
         """
         Create a new list (manual or smart). Corresponds to POST /lists.
@@ -1317,6 +1346,7 @@ class KarakeepAPI:
             parent_id: Optional parent list ID for nested lists.
             list_type: The type of list ('manual' or 'smart'). Default is 'manual'.
             query: Optional query string for smart lists (required if list_type is 'smart').
+            public: Whether the list is public (default: False).
 
         Returns:
             datatypes.ListModel: The created list object.
@@ -1336,6 +1366,7 @@ class KarakeepAPI:
             "name": name,
             "icon": icon,
             "type": list_type,
+            "public": public,
         }
 
         # Add optional fields if provided
@@ -1410,10 +1441,11 @@ class KarakeepAPI:
         icon: Optional[str] = None,
         parent_id: Optional[str] = None,
         query: Optional[str] = None,
+        public: Optional[bool] = None,
     ) -> Union[datatypes.ListModel, Dict[str, Any], List[Any]]:
         """
         Update a list by its ID. Corresponds to PATCH /lists/{listId}.
-        Allows updating various list fields including name, description, icon, parent relationship, and query.
+        Allows updating various list fields including name, description, icon, parent relationship, query, and public status.
 
         Args:
             list_id: The ID (string) of the list to update.
@@ -1422,6 +1454,7 @@ class KarakeepAPI:
             icon: Optional new icon for the list.
             parent_id: Optional new parent list ID (can be None to remove parent relationship).
             query: Optional new query string for smart lists (minimum 1 character).
+            public: Optional new public status for the list.
 
         Returns:
             datatypes.ListModel: The updated list object.
@@ -1444,6 +1477,8 @@ class KarakeepAPI:
             update_data["parentId"] = parent_id
         if query is not None:
             update_data["query"] = query
+        if public is not None:
+            update_data["public"] = public
 
         # Ensure at least one field is being updated
         if not update_data:
@@ -1935,3 +1970,125 @@ class KarakeepAPI:
         response_data = self._call("GET", "users/me/stats")
         # No Pydantic validation applied here as the spec defines a simple dict response
         return response_data
+
+    @optional_typecheck
+    def upload_a_new_asset(
+        self, file: str
+    ) -> Union[datatypes.Asset, Dict[str, Any], List[Any]]:
+        """
+        Upload a new asset file. Corresponds to POST /assets.
+
+        Args:
+            file: Path to the file to upload.
+
+        Returns:
+            datatypes.Asset: Details about the uploaded asset (assetId, contentType, size, fileName).
+            If response validation is disabled, returns the raw API response (dict/list).
+
+        Raises:
+            FileNotFoundError: If the specified file does not exist.
+            APIError: If the API request fails (e.g., unsupported file type, file too large).
+            pydantic.ValidationError: If response validation fails (and is not disabled).
+        """
+        import os
+        import mimetypes
+
+        # Validate file path exists
+        if not os.path.isfile(file):
+            raise FileNotFoundError(f"File not found: {file}")
+
+        # Get filename from path
+        file_name = os.path.basename(file)
+
+        # Detect MIME type
+        mime_type, _ = mimetypes.guess_type(file)
+        if mime_type is None:
+            mime_type = "application/octet-stream"
+
+        if self.verbose:
+            logger.debug(
+                f"Uploading asset: {file} (filename: {file_name}, type: {mime_type})"
+            )
+
+        # Prepare file for upload
+        try:
+            with open(file, "rb") as f:
+                file_content = f.read()
+            # Note: The 'file' key must match the OpenAPI spec parameter name
+            files = {"file": (file_name, file_content, mime_type)}
+            response_data = self._call("POST", "assets", files=files)
+        except IOError as e:
+            raise APIError(f"Failed to read file {file}: {e}") from e
+
+        if self.disable_response_validation:
+            logger.debug("Skipping response validation as requested.")
+            return response_data
+        else:
+            # Response should match Asset schema
+            return datatypes.Asset.model_validate(response_data)
+
+    @optional_typecheck
+    def get_a_single_asset(self, asset_id: str) -> bytes:
+        """
+        Get the raw content of an asset by its ID. Corresponds to GET /assets/{assetId}.
+
+        Args:
+            asset_id: The ID (string) of the asset to retrieve.
+
+        Returns:
+            bytes: The raw asset content. The Content-Type is determined by the asset type.
+                   Use response headers to determine the actual content type if needed.
+
+        Raises:
+            APIError: If the API request fails (e.g., 404 asset not found).
+            ValueError: If asset_id is empty or invalid.
+
+        Note:
+            This method always returns raw bytes regardless of the disable_response_validation setting,
+            as the response is binary content rather than JSON.
+        """
+        # Validate asset_id
+        if not asset_id or not asset_id.strip():
+            raise ValueError("asset_id cannot be empty")
+
+        asset_id = asset_id.strip()
+
+        # Validate asset_id format (basic check for reasonable ID format)
+        if len(asset_id) < 5:  # Assuming asset IDs are at least 5 characters
+            raise ValueError(f"asset_id appears to be invalid: {asset_id}")
+
+        endpoint = f"assets/{asset_id}"
+
+        # Override the Accept header to get raw content instead of JSON
+        # This is crucial for the assets endpoint to return binary data
+        extra_headers = {"Accept": "*/*"}
+
+        if self.verbose:
+            logger.debug(f"Retrieving asset: {asset_id}")
+
+        response_data = self._call("GET", endpoint, extra_headers=extra_headers)
+
+        # The _call method should return bytes for non-JSON responses when Accept is not application/json
+        if isinstance(response_data, bytes):
+            if self.verbose:
+                logger.debug(f"Retrieved asset {asset_id}: {len(response_data)} bytes")
+            return response_data
+        elif response_data is None:
+            # Handle empty response (valid for some assets like empty files)
+            if self.verbose:
+                logger.debug(f"Retrieved empty asset {asset_id}")
+            return b""
+        else:
+            # This shouldn't happen with the updated _call method, but handle gracefully
+            error_msg = f"Expected bytes from asset endpoint for asset {asset_id}, got {type(response_data).__name__}"
+            if isinstance(response_data, (dict, list)):
+                # If we got JSON, it might be an error response that wasn't caught
+                error_detail = (
+                    str(response_data)[:200] + "..."
+                    if len(str(response_data)) > 200
+                    else str(response_data)
+                )
+                error_msg += f". Response content: {error_detail}"
+
+            logger.error(error_msg)
+            raise APIError(error_msg)
